<?php


namespace Drupal\entity_reference_hierarchy_ief\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\inline_entity_form\Plugin\Field\FieldWidget\InlineEntityFormComplex;

/**
 * Plugin implementation of the 'entity_reference_hierarchy_autocomplete' widget.
 *
 * @FieldWidget(
 *   id = "inline_entity_form_tree",
 *   label = @Translation("Inline entity form"),
 *   field_types = {
 *     "entity_reference_hierarchy",
 *     "entity_reference_hierarchy_revisions",
 *   },
 *   multiple_values = true
 * )
 */
class InlineEntityFormHierarchy extends InlineEntityFormComplex {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $entities = $form_state->get(['inline_entity_form', $this->getIefId(), 'entities']);

    foreach ($entities as $key => $value) {
      $element['entities'][$key]['depth'] = [
        '#type' => 'textfield',
        '#size' => 3,
        '#title' => $this->t('Depth for row @number', ['@number' => $delta + 1]),
        '#title_display' => 'invisible',
        '#default_value' => !empty($items[$key]->depth) ? $items[$key]->depth : 0,
        '#weight' => 100,
        '#attributes' => [
          'class' => ['ief-entity-depth'],
        ],
      ];
    }

    $element['entities']['#entity_reference_hierarchy_ief'] = TRUE;

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $field_name = $this->fieldDefinition->getName();
    $field_value = $form_state->getValue($field_name);

    foreach ($values as $delta => $value) {
      if (isset($field_value['entities'][$delta])) {
        $values[$delta]['depth'] = $field_value['entities'][$delta]['depth'];
      }
    }

    return $values;
  }

}
