<?php

namespace Drupal\entity_reference_hierarchy;

/**
 * Class EntityReferenceHierarchyFieldItemListTrait
 */
trait EntityReferenceHierarchyFieldItemListTrait {

  /**
   * Returns a hierarchical outline array, keyed by field item delta.
   *
   * @return array
   */
  public function getFieldHierarchyOutline() {

    $parents = [-1];
    $prev_depth = 0;
    $tree = [];

    foreach ($this as $delta => $item) {
      $depth = $item->get('depth')->getValue();
      if ($depth > $prev_depth) {
        array_unshift($parents, $delta - 1);
      } elseif ($depth < $prev_depth) {
        array_splice($parents, 0, $prev_depth - $depth);
      }
      $prev_depth = $depth;

      $tree[$delta]['parent'] = $parents[0];
      $tree[$delta]['children'] = [];
    }

    $reverse_deltas = array_reverse(array_keys($this->getIterator()->getArrayCopy()));
    foreach ($reverse_deltas as $delta) {
      $parent_delta = $tree[$delta]['parent'];

      if ($parent_delta != -1) {
        $tree[$parent_delta]['children'] = [$delta => $tree[$delta]] + $tree[$parent_delta]['children'];
        unset($tree[$delta]);
      }
    }

    return $tree;

  }

}
